﻿unit androidfilepicker;

interface

uses
  System.SysUtils, System.Classes,
{$IFDEF ANDROID}
  Androidapi.JNIBridge, Androidapi.JNI.Net,
{$ENDIF ANDROID}
  System.SysConst, System.IOUtils, System.Messaging;

type
  TGetFileNameListener = procedure (const _ok: Boolean) of object;

  TAndroidFilePicker = class (TObject)
  private
    FLastPickedFileName : String;
    FFileNameListener   : TGetFileNameListener;
    FMessageChooserID   : Integer;
    {$IFDEF ANDROID}
    FLastPickedURI      : JNet_URI;

    function getBytesFromURI(_uri : Jnet_URI) : TBytes;
    function getURIfromMsg (m : TMessage) : Jnet_Uri;
    function trimExtension (_rawExt : string) : string;
    function getMimeType(_extension : string) : string;
    procedure InternalOpenFileDialog(_title, _extension:string);
    procedure HandleActivityMessageforChooser(const Sender: TObject; const M: TMessage);

    property fileNameCallBack : TGetFileNameListener read FFileNameListener;
    {$ENDIF}
  protected
  public
    constructor create(_fileNameCallBack: TGetFileNameListener);
    function getBytesFromlastPick : TBytes;
    {$IFDEF ANDROID}
    procedure OpenFileDialog(_title, _extension:string);
    property lastPickedURI : JNet_URI read FLastPickedURI;
    {$ENDIF}
    property lastPickedFileName : string read FLastPickedFileName;
  end;

implementation

uses
{$IFDEF ANDROID}
  Androidapi.Helpers, Androidapi.JNI.JavaTypes, Androidapi.JNI.GraphicsContentViewText,
  Androidapi.JNI.Webkit, Androidapi.JNI.App,
{$ENDIF}
  System.NetEncoding, System.Rtti, System.TypInfo, System.Math;


const
  FILE_SELECT_CODE = 0;

{$IFDEF ANDROID}

function TAndroidFilePicker.getBytesFromURI(_uri : Jnet_URI) : TBytes;
var
  jis: JInputStream;
  b: TJavaArray<Byte>;
begin
  if _uri <> nil then begin
    jis := TAndroidHelper.Context.getContentResolver.openInputStream(_uri);
    b := TJavaArray<Byte>.Create(jis.available);
    jis.read(b);
    jis.close;
    SetLength(result, b.Length);
    if b.Length > 0 then
      Move(b.Data^, result[0], b.Length);
  end else begin
    result := nil;
  end;
end;

procedure TAndroidFilePicker.HandleActivityMessageforChooser(const Sender: TObject; const M: TMessage);
var
  FileName  : string;
begin
  FLastPickedURI := getURIfromMsg(m);
  if FLastPickedURI = nil then begin
    FLastPickedFileName := '';
    exit;
  end else begin
    FLastPickedFileName := JStringToString(FLastPickedURI.getPath).Trim;
    if Assigned(fileNameCallBack) then
      fileNameCallBack(not FLastPickedFileName.IsEmpty);
  end;
end;

procedure TAndroidFilePicker.InternalOpenFileDialog(_title, _extension:string);
var
  Intent, IntentChooser : JIntent;
  sMIMEType, extension  : string;
begin
  extension := trimExtension(_extension);
  sMIMEType := getMimeType(extension);
  Intent := TJIntent.JavaClass.init(TJIntent.JavaClass.ACTION_GET_CONTENT);
  Intent.setType(StringToJString(sMIMEType));
  Intent.addCategory(TJIntent.JavaClass.CATEGORY_OPENABLE);
  IntentChooser := TJIntent.JavaClass.createChooser(Intent, StrToJCharSequence(_title));
  if FMessageChooserID <> 0 then
    TMessageManager.DefaultManager.Unsubscribe(TMessageResultNotification, FMessageChooserID);
  FMessageChooserID := 0;
  FMessageChooserID := TMessageManager.DefaultManager.SubscribeToMessage(
    TMessageResultNotification, HandleActivityMessageforChooser);
  try
    TAndroidHelper.Activity.startActivityForResult(IntentChooser, FILE_SELECT_CODE);
  except
    raise Exception.Create('Could not Find File Manager');
  end;
end;

Procedure TAndroidFilePicker.OpenFileDialog(_title, _extension:string);
begin
  InternalOpenFileDialog(_title, _extension);
end;

function TAndroidFilePicker.trimExtension (_rawExt : string) : string;
begin
  Result := _rawExt;
  if Result.Substring(0,1) = '.' then
    Result := Result.Substring(1);
  if Result  = '*.*' then
    Result := '*/*';
end;

function TAndroidFilePicker.getMimeType(_extension : string) : string;
var
  MIMEType              : JString;
  mime                  : JMimeTypeMap;
  sMIMEType, extension  : string;
begin
      extension := AnsiLowerCase(_extension);
  MIMEType := nil;
  sMIMEType := '';
  if extension = '' then
    sMIMEType := '*/*'
  else begin
    if extension = 'file/*' then
      sMIMEType := 'file/*';
    if extension = '*' then
    begin
      if TOSVersion.Check(4, 4) then
        sMIMEType := '*/*'
      else
        sMIMEType := 'file/*';
    end;
    if extension = '*/*' then
      sMIMEType := '*/*';
    if sMimeType = '' then begin
      mime := TJMimeTypeMap.JavaClass.getSingleton;
      if mime <> nil then
        MIMEType := mime.getMimeTypeFromExtension(StringToJString(extension));
      if MIMEType <> nil then
        sMIMEType := JStringToString(MIMEType).Trim;
    end;
  end;
  if sMIMEType.IsEmpty then
    sMIMEType := '*/*';
  result := sMIMEType;
end;

function TAndroidFilePicker.getURIfromMsg (m : TMessage) : Jnet_Uri;
begin
  Result := nil;
  if (m <> nil) and (M is TMessageResultNotification) then
    if (TMessageResultNotification(M).RequestCode = FILE_SELECT_CODE) then
      if (TMessageResultNotification(M).ResultCode = TJActivity.JavaClass.RESULT_OK) then
        if TMessageResultNotification(M).Value <> nil then
          result := TMessageResultNotification(M).Value.getData;
end;

{$ENDIF ANDROID}

function  TAndroidFilePicker.getBytesFromlastPick : TBytes;
begin
  {$IFDEF ANDROID}
  result := getBytesFromURI(lastPickedURI);
  {$ELSE}
  result :=nil;
  {$ENDIF}

end;

constructor TAndroidFilePicker.create(_fileNameCallBack: TGetFileNameListener);
begin
  inherited create;
  FFileNameListener := _fileNameCallBack;
  FMessageChooserID := 0;
end;


end.
