﻿unit filePicker;

interface
uses
{$IFDEF ANDROID}
  Androidapi.Jni.Net,
{$ENDIF}
  System.SysUtils, System.Types, FMX.Dialogs, androidfilepicker, mimetype;

type
  TNotifyPick = procedure(const _picked: Boolean) of object;

  TFilePicker = class (TObject)
  private
    FMimeMap : TMimeMap;
    FPickedFile : boolean;
    FPickedFileName : string;
    FCanReadExternalStorage : boolean;
    FCanWriteExternalStorage : Boolean;
    FOpenDialog: TOpenDialog;
    FDialogTitle : string;
    FNotifyPickCallback : TNotifyPick;
    Fpicker : TAndroidFilePicker;
    procedure GetFileName(const _ok: Boolean);
    function FileToByteArray(const _fileName: WideString): TArray<Byte>;
    function getPickedFileExtension : string;
    function getPickedFileMimeType  : String;
    function getPickedPureFileName  : String;
    function getPureFileName(_fileName : string) : string;
    function getLastSlashPos(_inStr : string) : integer;
  protected
  public
    constructor create(_openDLG : TOpenDialog; _title : string; _notifyPick : TNotifyPick);
    destructor Destroy; override;
    procedure openFileFicker(_extension : string);
    procedure updatePermissions;
    function getPickedFileBytes : TBytes;
    procedure resetPick;
    property hasPickedFile           : boolean            read FPickedFile;
    property pickedFileName          : string             read FPickedFileName;
    property pickedPureFileName      : string             read getPickedPureFileName;
    property pickedFileExtension     : string             read getPickedFileExtension;
    property pickedFileMimeType      : string             read getPickedFileMimeType;
    property canReadExternalStorage  : boolean            read FCanReadExternalStorage;
    property canWriteExternalStorage : boolean            read FCanWriteExternalStorage;
    property picker                  : TAndroidFilePicker read Fpicker;
    property mimeMap                 : TMimeMap           read FMimeMap;
  end;

implementation

uses
{$IFDEF ANDROID}
  Androidapi.Helpers, Androidapi.JNI.Os, Androidapi.Jni.JavaTypes,
{$ENDIF}
  System.Permissions, System.Classes;

//private

function TFilePicker.getLastSlashPos(_inStr : string) : integer;
var
  i: Integer;
begin
  result := 0;
  for i := length(_inStr) downto 1 do
    if _inStr[i] = '/' then begin
      result := i;
      break;
    end;
end;

function TFilePicker.getPickedPureFileName  : String;
begin
  result := getPureFileName(pickedFileName);
end;

function TFilePicker.getPureFileName(_fileName : string) : string;
var
  start, len : integer;
begin
  Result := StringReplace(_fileName, '\', '/', [rfReplaceAll, rfIgnoreCase]);
  start := getLastSlashPos(result);
  len := length(result) - start;
  result := result.Substring(start, len);
end;

function TFilePicker.getPickedFileExtension : string;
begin
  result := '';
  if hasPickedFile then
    result := mimeMap.getExtensionFromFileName(pickedFileName);
end;

function TFilePicker.getPickedFileMimeType  : String;
begin
  result := '';
  if hasPickedFile then
    result := mimeMap.getMimeTypeFromFileName(pickedFileName);
end;

procedure TFilePicker.GetFileName(const _ok: Boolean);
begin
  FPickedFile := _ok;
  FPickedFileName := picker.lastPickedFileName;
  FNotifyPickCallback(_ok);
end;

function TFilePicker.FileToByteArray(const _fileName : WideString): TArray<Byte>;
var
  fs : TFileStream;
begin
  fs:=TFileStream.Create(_fileName,fmOpenRead);
  try
    fs.Seek(0,soFromBeginning);
    setLength(result,fs.Size);
    fs.ReadBuffer(Pointer(result)^, fs.Size);
  finally
    fs.Free;
  end;
end;

// protected

procedure TFilePicker.updatePermissions;
{$IFDEF ANDROID}
var
  FPermissionREAD_EXTERNAL_STORAGE,
  FPermissionWRITE_EXTERNAL_STORAGE: string;
{$ENDIF ANDROID}
begin
  FCanReadExternalStorage := True;
  FCanWriteExternalStorage := True;
{$IFDEF ANDROID}
  FCanReadExternalStorage := False;
  FCanWriteExternalStorage := False;
  FPermissionREAD_EXTERNAL_STORAGE := JStringToString(TJManifest_permission.JavaClass.READ_EXTERNAL_STORAGE);//读取文件
  FPermissionWRITE_EXTERNAL_STORAGE := JStringToString(TJManifest_permission.JavaClass.WRITE_EXTERNAL_STORAGE);//写入文件
  PermissionsService.RequestPermissions([FPermissionWRITE_EXTERNAL_STORAGE, FPermissionREAD_EXTERNAL_STORAGE],
      procedure(const APermissions: TClassicStringDynArray; const AGrantResults: TClassicPermissionStatusDynArray)
      begin
        if (Length(AGrantResults) > 0 ) and (AGrantResults[0] = TPermissionStatus.Granted) then
        begin
          FCanReadExternalStorage := True;
          FCanWriteExternalStorage := True;
        end
        else if (Length(AGrantResults) > 1 ) and (AGrantResults[1] = TPermissionStatus.Granted) then
        begin
          FCanReadExternalStorage := True;
          FCanWriteExternalStorage := False;
        end
        else
        begin
          FCanReadExternalStorage := False;
          FCanWriteExternalStorage := False;
        end;
      end);
{$ENDIF ANDROID}
end;

// public

function TFilePicker.getPickedFileBytes : TBytes;
begin
  result := nil;
  if hasPickedFile then begin
    {$IFDEF ANDROID}
    result := picker.getBytesFromlastPick;
    {$ENDIF ANDROID}
    {$IF DEFINED(MSWINDOWS) OR DEFINED(MACOS)}
      result := FileToByteArray(FPickedFileName)
    {$ENDIF}
  end;
end;

constructor TFilePicker.create(_openDLG : TOpenDialog; _title : string; _notifyPick : TNotifyPick);
begin
  inherited create;
  FMimeMap := TMimeMap.create;;
  FOpenDialog := _openDLG;
  FDialogTitle := _title;
  FNotifyPickCallback := _notifyPick;
  Fpicker := TAndroidFilePicker.create(GetFileName);
  resetPick;;
end;

destructor TFilePicker.destroy;
begin
  FMimeMap.Free;
  Fpicker.Free;
end;

procedure TFilePicker.resetPick;
begin
  FPickedFile := false;
  FPickedFileName := '';
end;

procedure TFilePicker.openFileFicker(_extension : string);
var
  FileExt: string;
begin
  resetPick;
  FileExt := _extension.Trim;
  if FileExt.Substring(0,1) = '.' then
    FileExt := FileExt.Substring(1);
{$IFDEF ANDROID}
  picker.OpenFileDialog('Choose File', FileExt);
{$ENDIF ANDROID}
{$IF DEFINED(MSWINDOWS) OR DEFINED(MACOS)}
{$IF NOT defined(IOS)}
  FOpenDialog.Title := FDialogTitle;
  FOpenDialog.DefaultExt := FileExt;
  FOpenDialog.Filter := '(*.' + FileExt + ')|*.' + FileExt + '| (*.*)|*.*';
  FPickedFile := FOpenDialog.Execute;
  if hasPickedFile then begin
    FPickedFileName := FOpenDialog.FileName;
    FNotifyPickCallback(true);
  end;
{$ENDIF}
{$ENDIF}
end;

end.
